package oclplugin.logic;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ocl.examples.pivot.ExpressionInOCL;
import org.eclipse.ocl.examples.pivot.OCL;
import org.eclipse.ocl.examples.pivot.ParserException;
import org.eclipse.ocl.examples.pivot.helper.OCLHelper;
import org.eclipse.ocl.examples.pivot.manager.MetaModelManager;
import org.eclipse.ocl.examples.pivot.utilities.PivotEnvironment;
import org.eclipse.ocl.examples.pivot.utilities.PivotEnvironmentFactory;
import org.eclipse.ocl.examples.pivot.utilities.PivotUtil;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Profile;


public class OCLValidator {
	
	
	
	public void validate(Model model) {

		final Model myModel = model;


		EList<Constraint> constraints = new BasicEList<Constraint>(myModel.getOwnedRules());
		                           
		// Get all OCL rules from the model
		final TreeIterator<EObject> iterator = myModel.eAllContents();
		while(iterator.hasNext()){
		       EObject tempObject = iterator.next();
		                                  
		       if (tempObject instanceof NamedElement) {
		              NamedElement ne = (NamedElement) tempObject;
		                                         
		              if(ne.getNamespace() != null) {
		                     final EList<Constraint> temp = ne.getNamespace().getOwnedRules();
		                                                
		                     if(!constraints.containsAll(temp)){
		                           constraints.addAll(temp);
		                     }
		              }
		       }
		}
		                           
		// Get all OCL rules from the profiles applied to the model
		for(Profile profile : myModel.getAllAppliedProfiles()){
		final TreeIterator<EObject> profileIterator = profile.eAllContents();
		       while(profileIterator.hasNext()){
		              EObject tempObject = profileIterator.next();
		                                         
		              if (tempObject instanceof NamedElement) {
		                     NamedElement ne = (NamedElement) tempObject;
		                                                
		                     if(ne.getNamespace() != null) {
		                     final EList<Constraint> temp = ne.getNamespace().getOwnedRules();
		                                  
		                         if(!constraints.containsAll(temp)){
		                             constraints.addAll(temp);
		                         }
		                     }
		              }
		       }      
		}

		// Check if any OCL constraint is violated
		for (Constraint constraint : constraints) {
		    String constraintName = constraint.getName();
		                                  
		    if (constraint.getSpecification() instanceof OpaqueExpression) {
		        OpaqueExpression opaqueExpression = ((OpaqueExpression) constraint.getSpecification());

		              // Find the body named OCL
		              int indexOfOCLBody = -1;
		        for (int i = 0; i < opaqueExpression.getLanguages().size() && indexOfOCLBody == -1; i++) {
		            if (opaqueExpression.getLanguages().get(i).equals("OCL")) {
		                indexOfOCLBody = i;
		             }
		         }
		                                         
		    // If the body was found, continue the check
		         if (indexOfOCLBody != -1) {
		             final String expression = opaqueExpression.getBodies().get(indexOfOCLBody);
		             final Namespace contextObject = constraint.getContext();

		             ResourceSet resourceSet = new ResourceSetImpl();
		             Resource resource = resourceSet.getResource(myModel.eResource().getURI(), true);
		                                                
		             MetaModelManager metaModelManager = PivotUtil.getMetaModelManager(resource); 
		             PivotEnvironmentFactory envFactory = new PivotEnvironmentFactory(null, metaModelManager);
		             PivotEnvironment environment = envFactory.createEnvironment();
		                                                
		             OCL.initialize(null);
		                                                
		             OCL ocl = OCL.newInstance(environment);
		             OCLHelper oclHelper = ocl.createOCLHelper(contextObject); 
		                                                                                  
		                                                
		             oclHelper.setContext(contextObject.eClass());
		             try {
		             // Turn the expression into a query and evaluate it
		             ExpressionInOCL createQuery = oclHelper.createQuery(expression);
		             Object result = ocl.evaluate(contextObject, createQuery);
		             } catch (ParserException e) {
		             Diagnostic diagnostic = e.getDiagnostic();
		             System.out.println(diagnostic.getMessage());
		             }
		         }
		       }
		    }
		}

}
